@@ -181,7 +181,6 @@ tor_gzip_compress(char **out, size_t *out_len,
         /* In case zlib doesn't work as I think .... */
         if (stream->avail_out >= stream->avail_in+16)
           break;
-        /* Falls through. */
       case Z_BUF_ERROR:
         offset = stream->next_out - ((unsigned char*)*out);
         old_size = out_size;
@@ -207,7 +206,7 @@ tor_gzip_compress(char **out, size_t *out_len,
   }
  done:
   *out_len = stream->total_out;
-#ifdef OPENBSD
+#if defined(OpenBSD)
   /* "Hey Rocky!  Watch me change an unsigned field to a signed field in a
    *    third-party API!"
    * "Oh, that trick will just make people do unsafe casts to the unsigned
@@ -320,7 +319,6 @@ tor_gzip_uncompress(char **out, size_t *out_len,
         /* In case zlib doesn't work as I think.... */
         if (stream->avail_out >= stream->avail_in+16)
           break;
-        /* Falls through. */
       case Z_BUF_ERROR:
         if (stream->avail_out > 0) {
           log_fn(protocol_warn_level, LD_PROTOCOL,
