@@ -99,8 +99,8 @@
 #define LD_CHANNEL   (1u<<21)
 /** Scheduler */
 #define LD_SCHED     (1u<<22)
-/** Denial of Service mitigation. */
-#define LD_DOS       (1u<<23)
+/** Guard nodes */
+#define LD_GUARD     (1u<<23)
 /** Number of logging domains in the code. */
 #define N_LOGGING_DOMAINS 24
 
