@@ -10,6 +10,7 @@
 #include "test.h"
 
 #include "connection.h"
+#include "hs_common.h"
 #include "main.h"
 #include "microdesc.h"
 #include "networkstatus.h"
@@ -95,6 +96,13 @@ mock_connection_connect_sockaddr(connection_t *conn,
   return 1;
 }
 
+static int
+fake_close_socket(evutil_socket_t sock)
+{
+  (void)sock;
+  return 0;
+}
+
 static void
 test_conn_lookup_addr_helper(const char *address, int family, tor_addr_t *addr)
 {
@@ -124,6 +132,7 @@ test_conn_get_connection(uint8_t state, uint8_t type, uint8_t purpose)
 
   MOCK(connection_connect_sockaddr,
        mock_connection_connect_sockaddr);
+  MOCK(tor_close_socket, fake_close_socket);
 
   init_connection_lists();
 
@@ -148,12 +157,13 @@ test_conn_get_connection(uint8_t state, uint8_t type, uint8_t purpose)
   assert_connection_ok(conn, time(NULL));
 
   UNMOCK(connection_connect_sockaddr);
-
+  UNMOCK(tor_close_socket);
   return conn;
 
   /* On failure */
  done:
   UNMOCK(connection_connect_sockaddr);
+  UNMOCK(tor_close_socket);
   return NULL;
 }
 
@@ -192,6 +202,15 @@ test_conn_get_basic_teardown(const struct testcase_t *tc, void *arg)
 
     if (!conn->linked_conn->marked_for_close) {
       connection_close_immediate(conn->linked_conn);
+      if (CONN_IS_EDGE(conn->linked_conn)) {
+        /* Suppress warnings about all the stuff we didn't do */
+        TO_EDGE_CONN(conn->linked_conn)->edge_has_sent_end = 1;
+        TO_EDGE_CONN(conn->linked_conn)->end_reason =
+          END_STREAM_REASON_INTERNAL;
+        if (conn->linked_conn->type == CONN_TYPE_AP) {
+          TO_ENTRY_CONN(conn->linked_conn)->socks_request->has_finished = 1;
+        }
+      }
       connection_mark_for_close(conn->linked_conn);
     }
 
@@ -212,6 +231,14 @@ test_conn_get_basic_teardown(const struct testcase_t *tc, void *arg)
 
   if (!conn->marked_for_close) {
     connection_close_immediate(conn);
+    if (CONN_IS_EDGE(conn)) {
+      /* Suppress warnings about all the stuff we didn't do */
+      TO_EDGE_CONN(conn)->edge_has_sent_end = 1;
+      TO_EDGE_CONN(conn)->end_reason = END_STREAM_REASON_INTERNAL;
+      if (conn->type == CONN_TYPE_AP) {
+        TO_ENTRY_CONN(conn)->socks_request->has_finished = 1;
+      }
+    }
     connection_mark_for_close(conn);
   }
 
@@ -239,13 +266,9 @@ test_conn_get_rend_setup(const struct testcase_t *tc)
   rend_cache_init();
 
   /* TODO: use directory_initiate_command_rend() to do this - maybe? */
-  conn->rend_data = tor_malloc_zero(sizeof(rend_data_t));
   tor_assert(strlen(TEST_CONN_REND_ADDR) == REND_SERVICE_ID_LEN_BASE32);
-  memcpy(conn->rend_data->onion_address,
-         TEST_CONN_REND_ADDR,
-         REND_SERVICE_ID_LEN_BASE32+1);
-  conn->rend_data->hsdirs_fp = smartlist_new();
-
+  conn->rend_data = rend_data_client_create(TEST_CONN_REND_ADDR, NULL, NULL,
+                                            REND_NO_AUTH);
   assert_connection_ok(&conn->base_, time(NULL));
   return conn;
 
@@ -340,10 +363,7 @@ test_conn_get_rsrc_teardown(const struct testcase_t *tc, void *arg)
 static void *
 test_conn_download_status_setup(const struct testcase_t *tc)
 {
-  (void)tc;
-
-  /* Don't return NULL, that causes the test to fail */
-  return (void*)"ok";
+  return (void*)tc;
 }
 
 static int
@@ -528,7 +548,8 @@ test_conn_get_rend(void *arg)
   tt_assert(connection_get_by_type_state_rendquery(
                                             conn->base_.type,
                                             conn->base_.state,
-                                            conn->rend_data->onion_address)
+                                            rend_data_get_address(
+                                                      conn->rend_data))
             == TO_CONN(conn));
   tt_assert(connection_get_by_type_state_rendquery(
                                             TEST_CONN_TYPE,
@@ -655,7 +676,8 @@ test_conn_download_status(void *arg)
   dir_connection_t *conn4 = NULL;
   connection_t *ap_conn = NULL;
 
-  consensus_flavor_t usable_flavor = (consensus_flavor_t)arg;
+  const struct testcase_t *tc = arg;
+  consensus_flavor_t usable_flavor = (consensus_flavor_t)tc->setup_data;
 
   /* The "other flavor" trick only works if there are two flavors */
   tor_assert(N_CONSENSUS_FLAVORS == 2);
